Unit TestIRQ;
{ ========================================================================== }
{ =   Lynx Tecnologia Eletronica Ltda.                                     = }
{ =   Projeto:   Drivers / Windows 95                   Data: 14/nov/1998  = }
{ =   Programa:  Teste do driver VAD32D para Windows 95.                   = }
{ =   Mdulo:    TestIRQ                                                   = }
{ =   Autor:     Lauro                                                     = }
{ =   Descrio: Teste da aquisicao por interrupo.                       = }
{ =   Verso:    1.0                                                       = }
{ ========================================================================== }
{
  So aquisitados os canais de 0 at nCanais-1. Os canais pares so
  programados na faixa +/- 5 volts e os impares na faixa +/- 2.5 volts.
}
{-------------------------  Descrio da Interface  -------------------------}
Interface

Uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
     StdCtrls, Spin, Grids, ComCtrls, ExtCtrls, Buttons;


{ ===================================================== }
{ =       Declarao dos Tipos da Srie Temporal      = }
{ ===================================================== }

  Const NumEveMax = 32;

  Type
    TpDtHr = record
      Dia,Mes,Ano,HH,Mn,Sg,MS: word;
    end;
    FileNameType   = string [79];    { PathStr }
    SignalNameType = string [15];
    UnidadeType    = string [7];
    ComentarioType = string [30];

    WindowFFTType = (WdRetangular, WdHanning,  WdHamming,
                     WdTriangular, WdBlackman, WdKaiser);

    TipoAqType = (AqDMA, AqCPU, AqCepstrum, AqAntiCepstrum, AqCepsPot);

    OrigemSTType = (OrEnsaio, OrGeracao, OrCepstrum, OrAntiCepstrum,
                    OrCombArq1, OrCombArq2, OrCombArq3, OrCombArq4,
                    OrUniSer, OrUniPar, OrOpeAri, OrSel, OrMed, OrFiltro,
                    OrOpMFT, OrCepsPot, OrRainMult, OrPolar);

  Type
    ConfCanaisType = record
      Nome   : SignalNameType;   { Denominao do canal }
      Unidade: UnidadeType;      { Unidade do canal }
      LimMax : single;           { Lim. mximo da escala }
      LimMin : single;           { Lim. mnimo da escala }
      Usado  : boolean;          { Indica se ativo }
      Janela : word              { Indica Tamanho Janela }
    end;

    CanaisEntrada = array [0..31] of ConfCanaisType;
    EventosType   = array [1..NumEveMax] of longint;

    HeaderTempType = record
      DtHr      : TpDtHr;           { Data e hora do incio do ensaio }
      Comentario: ComentarioType;   { Comentrio do ensaio }
      CnAD      : CanaisEntrada;    { Configurao dos canais }
      NoCnAtivos: smallint;         { Nmero de canais ativos }
      RangeAD   : longint;          { Range do conversor A/D (ex. 1024) }
      FreqAmostr: single;           { Frequncia de amostragem }
      NoAmostras: longint;          { Nmero de amostras por canal }
      TipoAq    : TipoAqType;       { Tipo de aquisio: DMA ou CPU }
      Origem    : OrigemSTType;     { Origem da srie: Aquis., etc. }
      ArqOrigem1: FileNameType;     { Nome do arquivo de origem 1 }
      ComOrigem1: ComentarioType;   { Comentrio do arquivo de origem 1 }
      DtHr1     : TpDtHr;           { Data e hora do arq. origem 1 }
      ArqOrigem2: FileNameType;     { Nome do arquivo de origem 2 }
      ComOrigem2: ComentarioType;   { Comentrio do arquivo de origem 2 }
      DtHr2     : TpDtHr;           { Data e hora do arq. origem 2 }
      ExCom     : array [1..6] of
                    ComentarioType; { Outros comentrios }
      NoEventos : smallint;         { Nmero de eventos (AqDados) }
      Eventos   : EventosType;      { Tabela de eventos (instantes) }
    end;


{ ===================================================== }
{ =               Declarao de tipos                 = }
{ ===================================================== }

  Type
    TpCoefReta = array [0..1] of single;

  Type TdlgTestIRQ = class(TForm)
    gbParametros: TGroupBox;
    cbFreqAq    : TComboBox;
    cbNoAmost   : TComboBox;
    seNoCanais  : TSpinEdit;
    lblFreqAq   : TLabel;
    lblNoAmost  : TLabel;
    lblNoCanais : TLabel;
    pbAquisicao : TProgressBar;
    sgTabela    : TStringGrid;
    StatusBar   : TStatusBar;
    tmrUpdate   : TTimer;
    ckGravaST   : TCheckBox;
    panToolBar  : TPanel;
    btStart     : TBitBtn;
    btStop      : TBitBtn;
    btClose     : TBitBtn;
    Procedure FormCreate (Sender: TObject);
    Procedure FormClose (Sender: TObject; var Action: TCloseAction);
    Procedure btStartClick (Sender: TObject);
    Procedure btStopClick (Sender: TObject);
    Procedure seNoCanaisChange (Sender: TObject);
    procedure tmrUpdateTimer(Sender: TObject);
  private
    { Private declarations }
    fAtivo    : boolean;       { Indica ensaio ativo }
    FreqAq    : single;        { Frequncia de amostragem }
    nCanais   : word;          { Nmero de canais a aquisitar }
    nAmostras : longint;       { Nmero de amostras por canal }
    nAmsBuf   : word;          { Nmero de amostras por buffer por canal }
    vAng      : TpCoefReta;    { Coeficiente angular p/ conversar em volts }
    vLin      : TpCoefReta;    { Coeficiente linear p/ conversar em volts }

    iAmsEns   : longint;       { Nmero da amostra no ensaio }
    inTab     : longint;       { Limite (em ams) para atualizao da tabela }
    idTab     : longint;       { Intervalo (em ams) para atualizao da tabela }
    hdST      : HeaderTempType;{ Cabealho da srie temporal }
    aST       : file;          { Arquivo de srie temporal }
    fGravaST  : boolean;       { Indica se grava a serie temporal }
    TamBuf    : longint;       { Tamanho do buffer para gravao }

    Procedure EnableControls;
    Procedure FinalizaAquisicao;
    Procedure GravaCabecalho;
    Procedure IniciaAquisicao;
    Procedure MontaCabecalho;
    Procedure ProgramaCanais;
    Procedure UpdateTabela;
    Procedure UpdateTabelaFixa;
  public
    { Public declarations }
    nBuffers  : smallint;      { Nmero de buffers de aquisio alocados }
  end;


{ ===================================================== }
{ =                Variveis globais                  = }
{ ===================================================== }

  Var dlgTestIRQ: TdlgTestIRQ;


{-------------------------  Descrio da Implementao ----------------------}
Implementation
  Uses VAD32api;

{$R *.DFM}

{ ===================================================== }
{ =           Declarao de variveis locais          = }
{ ===================================================== }

  Const {--------- Tabela de cdigos de ganho das faixas de entrada do A/D --}
    TabGanho: array [0..7] of byte =
      (cBi5v0,  cBi2v5,  cBi1v0,  cBi0v5,
       cUni5v0, cUni2v5, cUni1v0, cUni0v5);

  Const {--------------------- Tabela de frequncias de amostragem (em Hz) --}
    cTabFreq: array [0..7] of single =
      (50, 60, 100, 120, 200, 240, 300, 500);

  Const {-------------- Tabela de nmero de amostras a aquisitar por canal --}
    cTabAms: array [0..6] of longint =
      (0, 1000, 2000, 5000, 10000, 20000, 50000);

   Var {-------------------------- Informao da aquisio por interrupo --}
     AcquiredData: TpAcquiredData;


{ ========================================================================== }
{ =                  Implementao do objeto TdlgTestIRQ                   = }
{ ========================================================================== }

{-------------------------------- TdlgTestIRQ: Trata criao do formulrio --}
Procedure TdlgTestIRQ.FormCreate (Sender: TObject);
  Var i   : smallint;
      sAux: string;
begin
  {-- Inicia combo box da frequncia de amostragem --}
  cbFreqAq.Clear;
  for  i := 0 to 7  do
  begin
    sAux := Format ('%.6g Hz', [cTabFreq [i]]);
    cbFreqAq.Items.Add (sAux);
  end;
  {-- Inicia combo box do nmero de amostras --}
  cbNoAmost.Clear;
  cbNoAmost.Items.Add ('2 bilhes');
  for  i := 1 to 6  do
    cbNoAmost.Items.Add (IntToStr (cTabAms [i]));

  cbFreqAq.ItemIndex    := 5;       { 240 Hz }
  cbNoAmost.ItemIndex   := 3;    { 5000 amostras }
  seNoCanais.Value      := 24;
  sgTabela.Cells [0, 0] := 'A/D';
  sgTabela.Cells [1, 0] := 'Faixa';
  sgTabela.Cells [2, 0] := 'Tenso (V)';
  StatusBar.Panels.Items [0].Text := '';
  StatusBar.Panels.Items [1].Text := '';
  fAtivo := false;
  EnableControls;
  UpdateTabelaFixa;
  {-- Coeficientes da converso dos canais na faixa +/- 5V --}
  vAng [0] := 5 / 32768.0;
  vLin [0] := 0;
  {-- Coeficientes da converso dos canais na faixa +/- 2.5V --}
  vAng [1] := 2.5 / 32768.0;
  vLin [1] := 0;
end;

{----------------------- TdlgTestIRQ: Trata fechamento da caixa de dilogo --}
Procedure TdlgTestIRQ.FormClose (Sender: TObject; var Action: TCloseAction);
begin
  if  fAtivo  then
    FinalizaAquisicao;
end;

{----------------------- TdlgTestIRQ: Trata boto para iniciar a aquisio --}
Procedure TdlgTestIRQ.btStartClick (Sender: TObject);
  Var Dummy: smallint;
begin
  if  not fAtivo
  then begin
    pbAquisicao.Position            := 0;
    StatusBar.Panels.Items [0].Text := '';
    StatusBar.Panels.Items [1].Text := 'Aquisitando...';
    fAtivo := true;
    EnableControls;
    UpdateTabelaFixa;
    VAD32D_ReadAI (0, Dummy);
    IniciaAquisicao;
  end;
end;

{------------------- TdlgTestIRQ: Trata boto para interromper a aquisio --}
Procedure TdlgTestIRQ.btStopClick (Sender: TObject);
begin
  if  fAtivo
  then begin
    FinalizaAquisicao;
    StatusBar.Panels.Items [1].Text := 'Aquisio interrompida manualmente.';
  end;
end;

{------------------------- TdlgTestIRQ: Atualiza habilitao dos controles --}
Procedure TdlgTestIRQ.EnableControls;
begin
  btStart.Enabled    := not fAtivo;
  btStop.Enabled     := fAtivo;
  cbFreqAq.Enabled   := not fAtivo;
  cbNoAmost.Enabled  := not fAtivo;
  seNoCanais.Enabled := not fAtivo;
  tmrUpdate.Enabled  := fAtivo;
end;

{----------------------- TdlgTestIRQ: Finaliza a aquisio por interrupo --}
Procedure TdlgTestIRQ.FinalizaAquisicao;
begin
  VAD32D_StopAcquisition;     { Encerra aquisio por interrupo }
  fAtivo := false;
  EnableControls;
  if  fGravaST
  then begin
    if  iAmsEns > nAmostras  then iAmsEns := nAmostras;
    CloseFile (aST);
  end;
end;

{-------------------------- TdlgTestIRQ: Grava cabealho da srie temporal --}
Procedure TdlgTestIRQ.GravaCabecalho;
begin
  if  not fGravaST  then exit;
  AssignFile (aST, 'TESTE.TEM');
  Rewrite (aST, 1);
  BlockWrite (aST, hdST, SizeOf (hdST));
  TamBuf := nAmsBuf * 2 * nCanais;
end;

{------------------------- TdlgTestIRQ: Inicia a aquisio por interrupo --}
Procedure TdlgTestIRQ.IniciaAquisicao;
  Var i: smallint;
      pOut: longint;
begin
  nCanais    := seNoCanais.Value;               { Nmero de canais a aquisitar }
  nAmostras  := cTabAms [cbNoAmost.ItemIndex];
  FreqAq     := cTabFreq [cbFreqAq.ItemIndex];  { Frequncia de amostragem por canal }
  fGravaST   := ckGravaST.Checked;
  ProgramaCanais;
  iAmsEns    := 0;
  idTab      := Round (FreqAq / 2);             { Atualiza tabela 2 vezes por segundo }
  inTab      := idTab;                          { Limite (em ams) para atualizao da tabela }

  {-- Inicia arquivo de srie temporal (AqDados) --}
  MontaCabecalho;
  GravaCabecalho;

  {-- Dispara a aquisio  --}
  VAD32D_StartAcquisition (FreqAq, nAmostras);
end;

{------------------------ TdlgTestIRQ: Monta o cabealho da srie temporal --}
Procedure TdlgTestIRQ.MontaCabecalho;
  Procedure SetCnAD (Cn: smallint; Sup, Inf: single);
  begin
      hdST.CnAD [Cn].Usado  := true;
      hdST.CnAD [Cn].LimMax := Sup;
      hdST.CnAD [Cn].LimMin := Inf;
    end;
  Var i : smallint;
      dt: TDateTime;
begin
  with  hdST  do
  begin
    dt := Now;    { Data/hora corrente }
    with  DtHr  do
    begin
      DecodeDate (dt, Ano, Mes, Dia);
      DecodeTime (dt, HH, Mn, Sg, MS);
    end;
    Comentario := 'VxD Test';
    for  i := 0 to 31  do with  CnAD [i]  do
    begin
      Nome    := 'Canal ' + IntToStr (i);
      Unidade := 'volts';
      LimMax  := 5;
      LimMin  := -5;
      Usado   := false;
      Janela  := 0;
    end;
    NoCnAtivos := nCanais;
    RangeAD    := 4096;
    FreqAmostr := FreqAq;
    NoAmostras := nAmostras;
    TipoAq     := AqCPU;
    Origem     := OrEnsaio;
    ArqOrigem1 := '';
    ComOrigem1 := '';
    ArqOrigem2 := '';
    ComOrigem2 := '';
    DtHr1      := DtHr;
    DtHr2      := DtHr;
    for  i := 1 to 6  do ExCom [i] := Comentario;
    NoEventos := 0;
  end;
  for  i := 0 to Pred (nCanais)  do
    if  (i mod 2) = 0  then
      SetCnAD (i, 5.0, -5.0)   { Canal com +/- 5.0 V }
    else
      SetCnAD (i, 2.5, -2.5);  { Canal com +/- 2.5 V }
end;

{--------------------------- TdlgTestIRQ: Programa a memria de canais A/D --}
{ neste exemplo a posio na memria de canais corresponde
  ao nmero do canal A/D
}
Procedure TdlgTestIRQ.ProgramaCanais;
  Var Canal: byte;
begin
  VAD32D_ClearCM;                       { Limpa a memria e canais }
  for  Canal := 0 to Pred (nCanais)  do
    if  (Canal mod 2) = 0  then
      VAD32D_WriteCM (Canal, cBi5v0)    { Faixa em +/- 5.0 V }
    else
      VAD32D_WriteCM (Canal, cBi2v5);   { Faixa em  +/- 2.5 V }
end;

{-------------------------- TdlgTestIRQ: Trata mudana do nmero de canais --}
Procedure TdlgTestIRQ.seNoCanaisChange (Sender: TObject);
begin
  UpdateTabelaFixa;
end;

{-------------------------- TdlgTestIRQ: Trata temporizao de atualizao --}
Procedure TdlgTestIRQ.tmrUpdateTimer (Sender: TObject);
  Var ieStatus  : byte;
      n, Size, i: longint;
      iCn       : smallint;
begin
  if  fAtivo
  then begin
    ieStatus := VAD32D_GetAcquiredData (AcquiredData);   { L ndice da aquisio }
    iAmsEns  := AcquiredData.iLast;
    if  (ieStatus <> ieNoError) or (iAmsEns >= nAmostras) and (nAmostras <> 0)
    then begin
      FinalizaAquisicao;
      case  ieStatus  of
        ieNoError       : StatusBar.Panels.Items [1].Text := 'Fim da Aquisio.';
        ieIntOverrun    : StatusBar.Panels.Items [1].Text := 'Interrupt overrun!';
        ieAD_Error      : StatusBar.Panels.Items [1].Text := 'Erro no A/D';
        ieBufOverrun    : StatusBar.Panels.Items [1].Text := 'Buffer overrun!';
        ieVPICD_Error   : StatusBar.Panels.Items [1].Text := 'VPICD error!';
        ieEndAcquisition: StatusBar.Panels.Items [1].Text := 'Fim da Aquisio!';
        else              StatusBar.Panels.Items [1].Text := 'Erro na aquisio!';
      end;
    end
    else begin
      if  (AcquiredData.nSampGot > 0) // (iAmsEns >= inTab) true
      then begin
        StatusBar.Panels.Items [0].Text := Format ('Amostra: %d', [iAmsEns]);
        if  nAmostras > 0  then
          pbAquisicao.Position := Round (100 * iAmsEns / nAmostras);
        UpdateTabela;
        Inc (inTab, idTab);
      end;
      if  fGravaST and (AcquiredData.nSampGot > 0)
      then begin
        Size := nCanais * 2 * AcquiredData.nSampGot;
        BlockWrite (aST, AcquiredData.UserBuf, Size);
      end;
    end;
  end;
end;

{--------------- TdlgTestIRQ: Atualiza a tabela com ltimo dado aquisitado --}
Procedure TdlgTestIRQ.UpdateTabela;
  Var Canal : smallint;
      iBuf  : longint;     { Indice no buffer }
      ic    : smallint;
      vVolts: single;
begin
  iBuf := Pred (AcquiredData.nSampGot) * nCanais;   { Primeiro canal }
  iBuf := 0;
  for  Canal := 0 to Pred (nCanais)  do
  begin
    ic     := Canal mod 2;
    vVolts := AcquiredData.UserBuf [iBuf+Canal] * vAng [ic] + vLin [ic];
    sgTabela.Cells [2, Canal+1] := Format ('%8.3f', [vVolts]);
  end;
end;

{------------------------------------- TdlgTestIRQ: Atualiza a tabela fixa --}
Procedure TdlgTestIRQ.UpdateTabelaFixa;
  Var Canal: word;
begin
  sgTabela.RowCount := seNoCanais.Value + 1;
  for  Canal := 0 to Pred (seNoCanais.Value)  do
  begin
    sgTabela.Cells [0, Canal+1] := Format (' %.2d', [Canal]);
    sgTabela.Cells [2, Canal+1] := '';
    if  (Canal mod 2) = 0  then
      sgTabela.Cells [1, Canal+1] := '+/- 5 V'
    else
      sgTabela.Cells [1, Canal+1] := '+/- 2.5 V';
  end;
end;

end.
