Unit VAD32api;
{ ========================================================================== }
{ =   Lynx Tecnologia Eletrnica Ltda.                                     = }
{ =   Projeto:   Drivers / Windows 95                   Data: 14/nov/1998  = }
{ =   Programa:  Teste do driver VAD32D para Windows 95.                   = }
{ =   Mdulo:    VAD32api                                                  = }
{ =   Autor:     Lauro                                                     = }
{ =   Descrio: Acesso s primitivas do driver VAD32D.VXD                 = }
{ =   Verso:    1.0                                                       = }
{ ========================================================================== }

{-------------------------  Descrio da Interface  -------------------------}
Interface
  Uses Windows;

{ ===================================================== }
{ =              Declarao de constantes             = }
{ ===================================================== }

  Const {---------- Constantes referentes a configurao padro das placas --}
    sbaCAD = $300;   { Endereo base padro da CAD12/32 }
    sirCAD = 5;      { Interrupo padro da CAD12/32 }

  Const {---------------------------------- ieXXXX: Interrupt error status --}
    ieNoError        = 0;   { No error }
    ieIntOverrun     = 1;   { Interrupt overrun }
    ieAD_Error       = 2;   { A/D error }
    ieBufOverrun     = 3;   { Buffer list overrun }
    ieVPICD_Error    = 4;   { VPICD cant virtualize IRQ }
    ieEndAcquisition = 5;   { End of acquisition }
    ieDriver         = 100; { Erro de acesso ao driver }
    ieAcqStop        = 101; { Aquisio no est ativa }
    ieBufSize        = 102; { Dimenso errada do buffer }

  Const {------------------ Codigos de ganho das entrada A/D da CAD12/32 --}
    cBi5v0   = $f0;      { -5.0 a 5.0 V }
    cUni5v0  = $70;      {  0.0 a 5.0 V }
    cBi2v5   = $e0;      { -2.5 a 2.5 V }
    cUni2v5  = $60;      {  0.0 a 2.5 V }
    cBi1v0   = $d0;      { -1.0 a 1.0 V }
    cUni1v0  = $50;      {  0.0 a 1.0 V }
    cBi0v5   = $b0;      { -0.5 a 0.5 V }
    cUni0v5  = $30;      {  0.0 a 0.5 V }


{ ===================================================== }
{ =     Declarao de tipos e variveis pblicas      = }
{ ===================================================== }

  Const
    cBufSize = 16384;     { Tamanho do buffer em bytes }

  Type {---------------------- Parmetro de sada da apiW32_GetAquiredData --}
    TpBufArray = array [0..cBufSize-1] of smallint;
    TpAcquiredData = record
      ieStatus: byte;       { Status da aquisio }
      nSamples: dword;      { Nmero de amostras aquisitadas por canal }
      nSampGot: dword;      { Mmero de amostras transferidas por canal }
      iLast   : dword;      { last sample time order in the user buffer }
      UserBuf : array [0..cBufSize-1] of smallint;
    end;


{ ========================================================================== }
{ =                     Primitivas do driver VAD32D                        = }
{ ========================================================================== }

  {-- Primitivas de controle de acesso e iniciao --}
  Function  VAD32D_W32_Open: WordBool; stdcall;
  Procedure VAD32D_W32_Close; stdcall;
  Function  VAD32D_GetVersion: word; stdcall;
  Function  VAD32D_InitCard (AbaCAD1232: word; AIntNo: byte): WordBool; stdcall;

  {-- Primitivas de gerais do driver --}
  Procedure VAD32D_SetGain (GainCode: byte); stdcall;
  Function  VAD32D_ReadAI (Channel: byte; Var Value: smallint): WordBool; stdcall;
  Function  VAD32D_ReadDI: word; stdcall;
  Function  VAD32D_ReadDO: word; stdcall;
  Procedure VAD32D_WriteDO (Value: word); stdcall;
  Procedure VAD32D_ProgTimer (Channel, Mode: byte; Count: word); stdcall;
  Function  VAD32D_ReadTimer (Channel: byte): word; stdcall;

  {-- Primitivas da aquisio por interrupo --}
  Procedure VAD32D_ClearCM; stdcall;
  Procedure VAD32D_WriteCM (Channel, GainCode: byte); stdcall;
  Function  VAD32D_StartAcquisition (AFreqAq: single; AnSamples: longint): WordBool; stdcall;
  Procedure VAD32D_StopAcquisition; stdcall;
  Function  VAD32D_GetAcquiredData (Var AcquiredData: TpAcquiredData): byte; stdcall;
  Function  VAD32D_GetSamples (Var ErrorCode, iSample, nSampRead: longint; BufSize: longint;
                               Var UserBuffer: TpBufArray): boolean; stdcall;


{--------------------------  Bloco de Implementao  ------------------------}
Implementation


{ ========================================================================== }
{ =              Primitivas de controle de acesso ao driver                = }
{ ========================================================================== }

{----------------------------------------- Carrega o driver VxD VAD32D.VXD --}
Function VAD32D_W32_Open; stdcall; external 'VAD32D.DLL';

{------------------------------------------- Fecha o driver VxD VAD32D.VXD --}
Procedure VAD32D_W32_Close; stdcall; external 'VAD32D.DLL';

{------------------- Devolve a verso do VAD32D: MSB: verso; LSB: reviso --}
Function VAD32D_GetVersion; stdcall; external 'VAD32D.DLL';

{--------------------------------------------- Inicia a placa A/D CAD12/32 --}
Function VAD32D_InitCard; stdcall; external 'VAD32D.DLL';

{--------------------------------- Programa o ganho de entrada da CAD12/32 --}
Procedure VAD32D_SetGain; stdcall; external 'VAD32D.DLL';

{------------------------------------------- L canal de entrada analgica --}
Function VAD32D_ReadAI; stdcall; external 'VAD32D.DLL';

{---------------- Leitura dos ports P0 e P1 de entrada digital da CAD12/32 --}
Function VAD32D_ReadDI; stdcall; external 'VAD32D.DLL';

{------------- Retorna ltimo valor escritos nos ports P0 e P1 da CAD12/32 --}
Function VAD32D_ReadDO; stdcall; external 'VAD32D.DLL';

{------------------ Escreve nos ports P0 e P1 de sada digital da CAD12/32 --}
Procedure VAD32D_WriteDO; stdcall; external 'VAD32D.DLL';

{-------------------------------------------- Programa o Timer da CAD12/32 --}
Procedure VAD32D_ProgTimer; stdcall; external 'VAD32D.DLL';

{------------------------- Escreve no port P0 demsada digital da CAD12/32 --}
Function VAD32D_ReadTimer; stdcall; external 'VAD32D.DLL';

{----------------------------------------------- Limpa a memria de canais --}
Procedure VAD32D_ClearCM; stdcall; external 'VAD32D.DLL';

{-------------------------------------- Programa memria de canais virtual --}
Procedure VAD32D_WriteCM; stdcall; external 'VAD32D.DLL';

{---------------------------------------- Inicia aquisio por interrupo --}
Function VAD32D_StartAcquisition; stdcall; external 'VAD32D.DLL';

{--------------------------------------- Encerra aquisio por interrupo --}
Procedure VAD32D_StopAcquisition; stdcall; external 'VAD32D.DLL';

{------------------------------------- L o nmero de amostras aquisitadas --}
{ Informa se h dados novos aquisitados. Veja ieXXXX. O status de erro      }
{ resetado aps a chamada desta primitiva.                                   }
Function VAD32D_GetAcquiredData; stdcall; external 'VAD32D.DLL';

{------------------------------------- Prepara a aquisio por interrupo --}
Function VAD32D_GetSamples; stdcall; external 'VAD32D.DLL';


{ ===================================================== }
{ =            Rotina de iniciao do mdulo          = }
{ ===================================================== }

end.

